/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.beanutils.BeanMap;

/**
 * @author Claes Lundahl, vgrid=clalu4
 * 
 */
public class Util {

    /**
     * Copies all properties from source object to target object.
     * 
     * @param source
     * @param target
     */
    public static void copyValuesFromBeanToBean(Object source, Object target) {
        new BeanMap(target).putAllWriteable(new BeanMap(source));
    }

    /**
     * Copies all properties from source object to target object. When a Set object is encountered, though, a new
     * instance of the type {@link HashSet} is created and the content from the target object is moved to that one
     * before being set into target.
     * 
     * @param source
     * @param target
     */
    public static void copyValuesAndSetsFromBeanToBean(Object source, Object target) {
        BeanMap sourceMap = new BeanMap(source);
        BeanMap targetMap = new BeanMap(target);

        new BeanMap(target).putAllWriteable(new BeanMap(source));

        for (Object key : targetMap.keySet()) {
            Object value = sourceMap.get(key);
            if (targetMap.getWriteMethod((String) key) != null) {
                if (value instanceof Set) {
                    Set newSet = new HashSet();
                    Set oldSet = (Set) value;
                    newSet.addAll(oldSet);
                    value = newSet;
                }

                targetMap.put(key, value);
            }
        }
    }

}
